/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.DeflaterInputStream;
import java.util.zip.GZIPInputStream;

import javax.swing.ImageIcon;

/**
 * Base class for stream directory implementations.
 * 
 * @author mikeL@launchpad.net
 */
public abstract class DirectoryFeed {

    /**
     * index of small icon (16x16px)
     */
    public final static int ICON_SMALL = 0;
    /**
     * index of large icon (32x32px)
     */
    public final static int ICON_LARGE = 1;

    private static final Map<String, Class<? extends DirectoryFeed>> registeredFeeds = new TreeMap<String, Class<? extends DirectoryFeed>>();

    public static DirectoryFeed getDirectoryFeed(String url) throws DirectoryFeedException
    {
        Class<? extends DirectoryFeed> feed = registeredFeeds.get(url);
        
        if (feed==null) throw new DirectoryFeedException("No DirectoryFeed implementation for \""+url+"\" available!",null);
        try
        {
            return feed.newInstance();
        }
        catch (Exception e)
        {
            throw new DirectoryFeedException("DirectoryFeed implementation for \""+url+"\" could not be instanciated!",null);
        }
    }

    protected static void registerDirectoryFeed(String url, Class<? extends DirectoryFeed> implClass) 
    {
        registeredFeeds.put(url, implClass);
    }

    public static String[] getURLsForRegisteredFeeds() {
        return registeredFeeds.keySet().toArray(new String[registeredFeeds.keySet().size()]);
    }

    public abstract String getName();

    /**
     * This method provides 2 icons (16x16 and 32x32).
     * 
     * @see DirectoryFeed#ICON_SMALL
     * @see DirectoryFeed#ICON_LARGE
     * @return an array containing 2 icons, 16x16 and 32x32
     */
    public abstract ImageIcon[] getImageIcons();

    public abstract List<String> getGenres() throws DirectoryFeedException;

    public abstract List<Station> getSearchResult(String searchTerm, Integer bitrate)
            throws DirectoryFeedException;

    public abstract List<Station> getStations(String genre, Integer bitrate)
            throws DirectoryFeedException;
    
    public abstract void flush();

    public abstract Station.Property[] getSupportedStationProperties();

    public boolean supportsStationProperty(Station.Property property)
    {
        List<Station.Property> properties = Arrays.asList(getSupportedStationProperties());
        return properties.contains(property);
    }

    protected InputStream getInputStreamForHttpUrl(URL url) throws IOException
    {
        if (!url.getProtocol().equals("http"))
            throw new IllegalArgumentException("Protocol must be http!");

        // TODO output debug messages to optional log file
        boolean debug = false;

        InputStream in = null;
        URLConnection conn = url.openConnection();
        // tell webserver that compressed content is accepted
        // saves time and traffic
        conn.setRequestProperty("Accept-Encoding", "gzip,deflate");

        if (debug)
        {
            // output request and response headers
            System.out.println("Debug Output for HTTP-Request");
            System.out.println("\n>>> Request-Header:");
            for (String header: conn.getRequestProperties().keySet())
                System.out.println("\t"+header+": "+conn.getRequestProperty(header));
            System.out.println("\n>>> Response-Header:");
            for (String header: conn.getHeaderFields().keySet())
                System.out.println("\t"+(header==null?"":header+": ")+conn.getHeaderField(header));
        }

        in = new BufferedInputStream(conn.getInputStream());
        // respect content encoding
        if ("gzip".equals(conn.getContentEncoding()))
            in = new GZIPInputStream(in);
        else if ("deflate".equals(conn.getContentEncoding()))
            in = new DeflaterInputStream(in);

        return in;
    }
}
