/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.SwingUtilities;

import org.kuehntopf.streamtastic.core.config.Config;
import org.kuehntopf.streamtastic.core.streamripper.StreamRipper;
import org.kuehntopf.streamtastic.core.streamripper.StreamRipperStatus;
import org.kuehntopf.streamtastic.utils.Player;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * This class manages StreamRipper instances
 * and sends Messages across the bus accordingly.
 * @author akuehntopf
 */
public class RippingManager extends Observable implements Observer  
{
    private List<Station> _currentRippedStations;
    private Map<String, StreamRipper> _urlToRipper;
    private Set<Integer> _freeRelayPorts;

    private static RippingManager _instance = null;   
    
    /**
     * Creates a new RippingManager instance.
     * This is private for Singleton reasons
     */
    private 
    RippingManager()
    {
        _currentRippedStations = new ArrayList<Station>();
        _urlToRipper = new HashMap<String, StreamRipper>();
        _freeRelayPorts = new TreeSet<Integer>();
        
        Config.getInstance().addPropertyChangeListener(Config.Key.STREAMRIPPER_RELAY_PORT_RANGE,
            new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt)
                {
                    updateRelayPortRange();
                }
            });
        updateRelayPortRange();
    }
    
    private void updateRelayPortRange()
    {
        _freeRelayPorts.clear();
        int[] ports= Config.getInstance().getStreamRipperRelayPortRange();
        if (ports != null)
            for (int i = ports[0]; i <= ports[1]; i++)
                _freeRelayPorts.add(i);
    }
    
    /**
     * Returns the singleton instance of this
     * class
     */
    public static RippingManager 
    getInstance()
    {
        if (_instance == null)
            _instance = new RippingManager();
        return _instance;
    }
    
    /**
     * Starts a new Download
     */
    public void 
    createRip(Station s)
    {
        if (isRipping(s) || s.getUrl() == null)
            return;
        
        // get free relay port
        Integer port = null;
        if (!_freeRelayPorts.isEmpty()) 
        {
            port = _freeRelayPorts.iterator().next();
            _freeRelayPorts.remove(port);
        }
        
        // Create a new Ripper and 
        // start it
        StreamRipper r = new StreamRipper(s, port);
        _currentRippedStations.add(s);
        _urlToRipper.put(s.getUrl(), r);
        r.addObserver(this);
        r.start();
        
        // Notify Observers of the newly added
        // download
        setChanged();
        notifyObservers(new CompoundMessage(Message.RIP_CREATED, s));        
    }         
    
    /**
     * Removes a Download
     */
    public void
    removeRip(Station s)
    {
        StreamRipper ripper = _urlToRipper.get(s.getUrl());
        if (ripper==null)
            return;

        boolean isOnRelayPlayBack = Player.isPlaying(ripper.getRelayUrl());
        ripper.stop();

        // switch back from relay, if current station was played
        if (isOnRelayPlayBack)
            try
            {
                Player.play(s.getUrl());
            }
            catch (UnsupportedOperationException e)
            {
                System.err.println(e.getMessage());
            }
    }

    /**
     * Removes all Downloads
     */
    public void
    removeAll()
    {
        // iterating over values() array copy to avoid ConcurrentModificationException
        for (StreamRipper ripper:_urlToRipper.values().toArray(new StreamRipper[_urlToRipper.size()]))
            ripper.stop();
    }

    /**
     * Update method called by the StreamRippers
     * This means the StreamRipper has a new status available
     * @param obs Observable
     * @param obj Objects
     */
    public void 
    update(Observable obs, Object obj)
    {       
        final StreamRipper rip = (StreamRipper)obs;
                
        // If no status has been set yet,
        // we cannot check anything, so just return       
        if (rip.getStatus() == null)
            return;
        
        // switch to local relay, if current station is played
        if (rip.getStatus() == StreamRipperStatus.BUFFERING)
            if (Player.isPlaying(rip.getStation().getUrl()) && rip.getRelayUrl() != null)
                try
                {
                    Player.play(rip.getRelayUrl());
                }
                catch (UnsupportedOperationException e)
                {
                    System.err.println(e.getMessage());
                }

        // If terminated, check the reason
        if (rip.getStatus().equals(StreamRipperStatus.TERMINATED))
        {
            // If it got terminated by error: Show it
            if (rip.getError() != null)
                SwingUtilities.invokeLater(new Runnable() { public void run()
                {
                    Utilities.showError(String.format(Utilities
                                .getString("STREAMRIPPER_TERMINATED"), rip.getError(), rip
                                .getStation().getUrl()));
                }});
            
            // Has been terminated, remove it
            // and exit this method to prevent sending two messages
            _currentRippedStations.remove(rip.getStation());
            _urlToRipper.remove(rip.getStation().getUrl());
            if (rip.getRelayPort() != null)
                _freeRelayPorts.add(rip.getRelayPort());
            setChanged();
            notifyObservers(new CompoundMessage(Message.RIP_REMOVED, rip.getStation()));

            return;
        }      
        
        // Notify observers
        setChanged();
        notifyObservers(new CompoundMessage(Message.RIP_UPDATED, rip.getStation()));
    }
    
    /**
     * Checks if a Station is already being ripped
     */
    public boolean
    isRipping(Station s)
    {
        return _urlToRipper.containsKey(s.getUrl());
    }
    
    protected StreamRipper getRipperFor(Station s)
    {
        return _urlToRipper.get(s.getUrl());
    }

    /**
     * Returns a list of all ripped stations
     * @return stations array
     */
    public Station[] 
    getRippedStations()
    {
        return _currentRippedStations.toArray(new Station[_currentRippedStations.size()]);
    }
}
