/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.HashMap;
import java.util.Map;

import org.kuehntopf.streamtastic.core.streamripper.StreamRipper;
import org.kuehntopf.streamtastic.core.streamripper.StreamRipperStatus;

/**
 * Station base class
 * @author akuehntopf
 */
public class Station 
{
    public static enum Property
    {
        NAME, CURRENT_TRACK, LISTENERS, MIMETYPE, BITRATE, GENRE, URL,
        RECORDING_STATE, RECORDED_DATA_AMOUNT;

        private static final Map<Property, String> propertyMap = new HashMap<Property, String>();
        static
        {
            propertyMap.put(NAME, "name");
            propertyMap.put(CURRENT_TRACK, "currentTrack");
            propertyMap.put(LISTENERS, "listeners");
            propertyMap.put(MIMETYPE, "mimetype");
            propertyMap.put(BITRATE, "bitrate");
            propertyMap.put(GENRE, "genre");
            propertyMap.put(URL, "url");
            propertyMap.put(RECORDING_STATE, "recordingState");
            propertyMap.put(RECORDED_DATA_AMOUNT, "recordedDataAmount");
        }

        @Override
        public String toString()
        {
            return propertyMap.get(this);
        }
    }

    static {
        // make recording a transient bean property for the XMLEncoder
        // this solves problems which occured when a currently recorded station was added to favorites
        try
        {
            BeanInfo beanInfo = Introspector.getBeanInfo(Station.class);
            for (PropertyDescriptor property : beanInfo.getPropertyDescriptors())
                if (property.getName().equals("recording"))
                    property.setValue("transient", true);
        }
        catch (IntrospectionException e)
        {
            System.err.println("Switching \"recording\" to a transient property failed!");
        }
    }

    private String      _name;
    private String       _mt;
    private String      _url;
    private Integer     _bitrate;
    private String       genreString;
    private String      _currentTrack;
    private Integer  _listeners;

     
    /**
     * Constructs a new Station
     */
    public 
    Station()
    {        
    }
    
    /**
     * Constructs a new Station with parameters
     * @param name Name of the Station
     * @param url URL of the Station
     */
    public 
    Station(String name, String url)
    {
        _name = name;
        _url = url;
    }
       
    /**
     * Returns the Station's name
     * @return the name of the station
     */
    public String 
    getName() 
    {
        return _name;
    }

    /**
     * Sets this Station's name
     * @param name
     */
    public void 
    setName(String name) 
    {
        this._name = name;
    }

    /**
     * Returns if the Station is recording
     * @return recording
     */
    public boolean 
    isRecording() 
    {
        return RippingManager.getInstance().isRipping(this);
    }
    
    public void 
    setRecording(boolean recording) 
    {
        if (recording)
            RippingManager.getInstance().createRip(this);
        else 
            RippingManager.getInstance().removeRip(this);
    }

    public StreamRipperStatus
    getRecordingState()
    {
        StreamRipper ripper = RippingManager.getInstance().getRipperFor(this);
        if (ripper == null)
            return null;
        
        return ripper.getStatus();
    }

    public String 
    getRecordedDataAmount() 
    {
        StreamRipper ripper = RippingManager.getInstance().getRipperFor(this);
        if (ripper == null)
            return null;

        return ripper.getTransferred();
    }

    /**
     * Returns the Station's url
     * @return the url of the station
     */
    public String 
    getUrl() 
    {
        return _url;
    }

    public String
    getRelayUrl()
    {
        if (!isRecording()) 
            return null;
        return RippingManager.getInstance().getRipperFor(this).getRelayUrl();
    }

    /**
     * Sets this Station's url
     * @param url
     */
    public void 
    setUrl(String url) 
    {
        // setRecording(false) won' t work if a new url would be set while ripping, thus throw exception
        if (this.isRecording())
            throw new IllegalStateException(
                    "setting the url while station is ripped is not allowed!");
        this._url = url;
    }     
    
    /**
     * Returns a String representation 
     * of this Station
     * @return string representation
     */
    @Override
    public String 
    toString()
    {
        return getName();
    }

    /**
     * Returns the Station's mimetype
     * @return the the mimetype of the Station
     */
    
    public String getMimetype()
    {
        return _mt;
    }

    /**
     * Sets this Station's mimetype
     * 
     * @param mimetype
     */

    public void setMimetype(String _mt)
    {
        this._mt = _mt;
    }

    /**
     * Returns the Station's bitrate
     * 
     * @return the bitrate of the station
     */

    public Integer getBitrate()
    {
        return _bitrate;
    }

    /**
     * Sets this Station's bitrate
     * 
     * @param bitrate
     */

    public void setBitrate(Integer _bitrate)
    {
        this._bitrate = _bitrate;
    }

    /**
     * Sets this Station's genre
     * 
     * @param genre
     */

    public void setGenre(String genreString)
    {
        this.genreString = genreString;
    }

    /**
     * Returns the Station's genre
     * 
     * @return the genre of the station
     */
    public String getGenre()
    {
        return genreString;
    }

    /**
     * Returns the Station's current track
     * 
     * @return the current track of the station
     */

    public String getCurrentTrack()
    {
        //update current track while ripping
        if (isRecording()
                && (getRecordingState() == StreamRipperStatus.RIPPING || 
                    getRecordingState() == StreamRipperStatus.SKIPPING)) 
        {
            StreamRipper ripper = RippingManager.getInstance().getRipperFor(this);
            if (ripper!=null)
                _currentTrack = ripper.getTrack();
        }
        return _currentTrack;
    }

    /**
     * Sets this Station's current Track
     * 
     * @param track
     */

    public void setCurrentTrack(String track)
    {
        _currentTrack = track;
    }

    /**
     * Returns the Number of Listeners for this Station
     * 
     * @return number of listeners
     */
    public Integer getListeners()
    {
        return _listeners;
    }

    /**
     * Sets the number of listeners for this station
     * 
     * @param listeners
     *            number of listeners
     */
    public void setListeners(Integer listeners)
    {
        this._listeners = listeners;
    }
    
    /**
     * Compares this Station to others
     * @param obj Station to compare to
     * @return are both Stations equal?
     */
    @Override
    public boolean 
    equals(Object obj) 
    {
        if (obj instanceof Station)
        {
            Station s = (Station)obj;
            return s.getUrl().equals(this.getUrl());
        } else
        {
            return super.equals(obj);
        }
    }
    
    /**
     * Returns the HashCode for this Station
     * This is required for correct equals behavior.
     * @return hashcode of this Object
     */
    @Override
    public int 
    hashCode() 
    {
        int hash = 5;
        hash = 47 * hash + (this._url != null ? this._url.hashCode() : 0);
        return hash;
    }            
}