/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core.streamripper;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import org.kuehntopf.streamtastic.core.config.Config;
import org.kuehntopf.streamtastic.utils.Command;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class StreamripperCommand extends Command {

    public static final String PARAMETER_DEST_DIR= Option.DESTINATION_DIRECTORY.PARAMETER;
    public static final String PARAMETER_RELAY_PORT= Option.CREATE_RELAY_STREAM.PARAMETER;
    public static final String PARAMETER_URL= Option.STREAM_URL.PARAMETER;

    public static enum Option {
        // CONNECTION SETTINGS
        STREAM_URL("","url"),
        PROXY("-p","proxy"), 
        USER_AGENT("-u", "useragent"), 
        DISABLE_RECONNECT("-c",null), 
        STREAM_STUTTER_TIMEOUT("-m","stutterTimeout"), 
        MAX_RELAY_STREAM_CONNECTIONS("-R","maxRelayConnections"),
        // -r won't work without -z on windows, this seems to be a streamripper bug
        // nevertheless -z is always used to ensure that the given port is the relay port
        // otherwise this could lead to trouble if we try to determine the relay url
        CREATE_RELAY_STREAM("-z_-r", "relayPort"), 

        // OUTPUT-OPTIONS
        DESTINATION_DIRECTORY("-d","destinationDir"),
        NO_SEPARATE_DIRECTORY_PER_STATION("-s",null),
        DONT_CREATE_TRACKS("-A",null), 
        SKIP_TRACKS_ON_START("-k","skippedTracks"),
        TRACK_PADDING("--xs-padding=","trackPadding"), 
        RIP_TO_SINGLE_FILE("-a","singleFilePattern",true),
        FILENAME_PATTERN("-D","fileNamePattern"),
        ADD_SEQ_NUMBER_TO_TRACKS("-q","sequenceNumber",true),

        // HANDLING_DUPLICATES
        OVERWRITE_COMPLETE_TRACKS("-o","overwriteMode"), // [always, never, larger, version]
        DONT_OVERWRITE_TRACKS_IN_INCOMPLETE("-t",null),
        TRUNCATE_TRACKS_IN_INCOMPLETE("-T",null),

        // ID3 TAGS
        DONT_CREATE_ID3_TAGS("-i",null),
        ADD_ID3V1("--with-id3v1",null),
        NO_ID3V2("--without-id3v2",null),

        // STOP CONDITIONS
        TIME_LIMIT("-l","timeLimit"),
        SPACE_LIMIT("-M","spaceLimit"),

        // META DATA PROCESSING
        META_DATA_PARSING_RULES("-w","patternFile"),
        META_DATA_PROVIDER("-E","metaDataProvider"),

        // SILENCE DETECTION
        NO_SILENCE_DETECTION("--xs-none", null),
        ALTERNATIVE_SILENCE_DETECTION("--xs2", null),
        MIN_SILENCE_LENGTH_FOR_SPLIT("--xs-silence-length=","minSilenceTime"),
        SILENCE_SEARCH_WINDOW("--xs-search-window=","silenceSearchWindow"),
        SILENCE_DETECTION_OFFSET_ON_TRACK_CHANGE("--xs-offset=","silenceDectionOffset"); 

        // currently NOT SUPPORTED:
        //   --codeset-*=,  --stderr, --quiet, --debug

        private static Map<String, Option> stringToOption = new HashMap<String, Option>();
        static
        {
            for (Option o : values())
            {
                // add streamRipperOption -> Option
                if (!(o == CREATE_RELAY_STREAM) || o.streamRipperOption.length() == 0)
                    stringToOption.put(o.streamRipperOption, o);
                // add toString() -> Option for streamRipperOptions ending with '='
                if (o.streamRipperOption.endsWith("=")) stringToOption.put(o.toString(), o);
                // add ${parameter} -> Option for options with parameter
                if (o.PARAMETER != null) stringToOption.put("${"+o.PARAMETER+"}", o);
            }
            // add -z -> CREATE_RELAY_STREAM, -r -> CREATE_RELAY_STREAM
            for (String s : CREATE_RELAY_STREAM.streamRipperOption.split("_"))
                stringToOption.put(s, CREATE_RELAY_STREAM);
        }
        public final String PARAMETER;
        private final String streamRipperOption;
        private final boolean optional;

        private Option(String streamRipperOption, String parameter)
        {
            this(streamRipperOption,parameter,false);
        }

        private Option(String streamRipperOption, String parameter, boolean optional)
        {
            PARAMETER = parameter;
            this.streamRipperOption = streamRipperOption;
            this.optional = PARAMETER!=null && optional;
        }

        public String toString()
        {
            // TODO underscore handling is inconsistent
            // underscore in ${...} is considered as splitchar
            // underscore before ${...} is ignored
            // examples: 1) ${-z_-r_relayPort} -> "-z -r port"
            //           2) --xs_padding=${padding} -> "--xs_padding=before:after"
            // currently no buggy behaviour, because there is no option containing _ used inside ${..}
            // UPDATE: streamripper allows substitution of _ through - thus we always use -
            //         example: --xs-padding=0:9 instead of --xs_padding=0:9
            if (streamRipperOption.endsWith("=")) return streamRipperOption+"${"+PARAMETER+"}";
            if (optional) return streamRipperOption +" ${"+PARAMETER+"}";
            if (streamRipperOption.length()==0) return "${"+PARAMETER+"}"; // affects only STREAM_URL
            return PARAMETER==null? streamRipperOption: "${"+streamRipperOption+"_"+PARAMETER+"}";
        }

        public String toString(String parameterValue)
        {
            // TODO value validation missing - where is best place??
            if (PARAMETER==null || parameterValue == null) return toString();
            return replaceParameter(toString(),PARAMETER,parameterValue);
        }

        public static Option get(String value)
        {
            return stringToOption.get(value);
        }
    }

    /**
     * @param executable
     * @param options
     * @throws FileNotFoundException
     */
    public StreamripperCommand(String executable, String options) throws FileNotFoundException
    {
        //file exists check is bypassed
        //otherwise 'streamripper' won't be accepted as command, even if it's available from path
        //further checks are done by the validator which calls getVersionstatus()
        super(executable, options, false);
    }

    /**
     * @param options
     * @throws FileNotFoundException
     */
    public StreamripperCommand(String options) throws FileNotFoundException
    {
        this(Config.getInstance().getProperty(Config.Key.STREAMRIPPER_CMD), options);
    }

    protected StreamripperCommand(String executable, String options, boolean checkCanExecute)
            throws FileNotFoundException
    {
        super(executable, options, checkCanExecute);
    }

    /**
     * Checks if Streamripper is actually available
     * by trying to create a process. And getting its version.
     * @return the version of the streamripper application or null
     */
    public ProgramVersionStatus getVersionStatus()
    {        
        try {
            final Process p = execute("-v");
            //kill process if we have to wait longer than 5 seconds for output on stdout
            Thread watchdog = new Thread( new Runnable() { public void run() { 
                try 
                {
                    Thread.sleep(5000);
                    p.destroy();
                } 
                catch (InterruptedException e) {}
            }});
            watchdog.start();

            BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line = reader.readLine();
            watchdog.interrupt();
    
            // If nothing has been returned it means that this version
            // does not supply version information correctly on stdout.
            // This could mean its an older version which is not supported
            // anymore.
            if (line == null)
                return ProgramVersionStatus.NOTSUPPORTED;

            if (!line.matches(".*[Ss]tream[Rr]ipper.*"))
                return ProgramVersionStatus.NOTSUPPORTED;

            String version = line.split(" ")[1];
    
            // Further checks on the version of the executable go here.
            String[] parts = version.split("[.-]");
            try
            {
                int major = Integer.valueOf(parts[0]);
                int minor = Integer.valueOf(parts[1]);
                if (major < 1) // exclude 0.x
                    return ProgramVersionStatus.NOTSUPPORTED;
                if (major == 1 && minor < 64) // exclude 1.63 and older
                    return ProgramVersionStatus.NOTSUPPORTED;
                if (major == 1 && minor == 64 && version.matches(".*64(?:-beta.*|\\.[1-4]).*")) 
                    // exclude 1.64.4 and earlier
                    return ProgramVersionStatus.NOTSUPPORTED;
            }
            catch (RuntimeException e) // number format or index out of bounds
            {
                // version parsing failed
                // we accept anyway, so an errorneous detection algorithm is not
                // stopping streamtastic from working with future version formats
                System.err.println("Version detection failed for: " + version);
                System.err.println("Ensure you are using Streamripper "+getMinRequiredVersion()+" or later!");
                return ProgramVersionStatus.OK;
            }
    
            // accept 1.64.5 and above
            return ProgramVersionStatus.OK;
            
        } catch (IOException ex) {
            return ProgramVersionStatus.NOTAVAILABLE;
        }
    }

    public static String getMinRequiredVersion()
    {
        return "1.64.5";
    }

    public static boolean checkStreamripperOptions(String options)
    {
        try
        {
            Command testCmd = new StreamripperCommand(null, options, false);
            for (String parameter : new String[] { PARAMETER_DEST_DIR, PARAMETER_RELAY_PORT,
                    PARAMETER_URL })
                testCmd.set(parameter, null);
            return true;
        }
        catch (Exception e)
        {
            return false; 
        }
    }

    public static String compileOptionString(Map<Option, String> options)
    {
        StringBuffer optionString = new StringBuffer();

        //TODO process map, exclusion of colliding options

        // stream url comes always first
        optionString.append(Option.STREAM_URL.toString(options.get(Option.STREAM_URL)));
        for (Option option : options.keySet())
        {
            if (option == Option.STREAM_URL) continue;

            String optionS = option.toString(options.get(option));
            if (optionS.length()==0) continue;

            optionString.append(" ");
            optionString.append(optionS);
        }

        return optionString.toString();
    }

    public static Map<Option, String> decompileOptionString(String optionString)
            throws IllegalArgumentException
    {
        Map<Option, String> options = new TreeMap<Option, String>();

        Option lastOption = Option.STREAM_URL;
        for (String current : decomposeOptionString(optionString)) 
        {
            // we ignore -z, because it will be always present if -r is set, -z alone makes no sense
            // see comments to Option.CREATE_RELAY_STREAM for further infos on that
            if (current == null || current.length() == 0 || current.equals("-z")) continue;
            String value = null;

            // o != null for options like ${parameter}, --option=${parameter}, -option
            Option o = Option.get(current);

            if (o == null)
            {
                if (current.contains("="))
                // handle options like --option=value
                {
                   int olength = current.indexOf('=')+1;
                   o = Option.get(current.substring(0, olength));
                   value = current.substring(olength);
                }
                else 
                // current is the corresponding value for lastOption
                {
                    o = lastOption;
                    value = current;
                }
            }
            // o is still null, don't know how to handle current -> throw exception
            if (o==null) throw new IllegalArgumentException("OptionString contains unknown symbol: "+current);
            // a required value for lastOption is missing, break and throw exception
            if (lastOption!=null && !lastOption.optional && lastOption != o) break; 

            // add <option,value> to map, special handling for optional values neccessary
            options.put(o, (o.optional && lastOption!=o)?"":value);
            // set lastOption to o, if the next string may or must be a value for this option
            lastOption = (o.PARAMETER==null || value !=null || current.contains("${"))?null:o;
        }
        if (lastOption!=null) throw new IllegalArgumentException("Required value missing for:"+lastOption.streamRipperOption);

        return options;
    }

    public static File showSelectStreamripperDialog(Component parent) 
    {
        String location = Config.getInstance().getProperty(Config.Key.STREAMRIPPER_CMD);
        if (location == null) location = Utilities.getHome();

        JFileChooser filechooser = new JFileChooser(location);
        filechooser.setAcceptAllFileFilterUsed(false);
        filechooser.setDialogTitle(Utilities.getString("STREAMRIPPER_SELECTION_DLG_TITLE"));
        filechooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        filechooser.setFileFilter(new FileFilter() {
            @Override
            public boolean accept(File file)
            {
                if (file.isDirectory() || (file.getName().contains("streamripper"))
                        && file.canExecute())
                    return true;
                return false;
            }

            @Override
            public String getDescription()
            {
                return Utilities.getString("STREAMRIPPER_SELECTION_DLG_FILETYPE");
            }
        });
        filechooser.showDialog(parent, Utilities.getString("STREAMRIPPER_SELECTION_DLG_APPROVE_LABEL"));

        return filechooser.getSelectedFile();
    }
}
