/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */


package org.kuehntopf.streamtastic.gui.views;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JPanel;

import org.jdesktop.swingx.JXHeader;
import org.jdesktop.swingx.painter.CompoundPainter;
import org.jdesktop.swingx.painter.GlossPainter;
import org.jdesktop.swingx.painter.PinstripePainter;

/**
 * Abstract base class of a View
 * Views have to implement a activate() method
 * which gets called when the View is activated by clicking
 * the appropriate item in the menu
 * @author Andreas Kuehntopf
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
*/
public abstract class View extends JPanel
{
    private JXHeader header;
    private JComponent content;
    private Runnable task;
    private Icon displayIcon;

    public View(String title, Icon headerIcon, Icon displayIcon)
    {
        super (new BorderLayout());
        this.displayIcon = displayIcon;

        // TODO add description (sub caption), add methods setTitle(), setSubTitle()
        header = new JXHeader(title, null, headerIcon); 
        GlossPainter gloss = new GlossPainter(new Color(1.0f, 1.0f, 1.0f, 0.5f));
        PinstripePainter printstripes = new PinstripePainter(new Color(0.1f, 0.1f, 0.1f, 0.2f));
        printstripes.setSpacing(5);
        header.setBackgroundPainter(new CompoundPainter(printstripes, gloss));
        header.setOpaque(false);
        this.add(header, BorderLayout.NORTH);
        header.setTitleFont(header.getFont().deriveFont(Font.BOLD, 17f));
        header.setDescriptionFont(header.getFont().deriveFont(Font.BOLD));
        // the property change listener is necessary to restore fonts after look and feel changed
        // this must be a bug, because the fonts are still set correctly, they are just ignored
        // TODO reevaluate with new versions of swingx lib
        header.addPropertyChangeListener("UI", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt)
            {
                header.setTitleFont(null);
                header.setDescriptionFont(null);
                header.setTitleFont(getFont().deriveFont(Font.BOLD, 17f));
                header.setDescriptionFont(getFont().deriveFont(Font.BOLD));
            }
        });

        // for view activation and execution of pending task which requires user interaction
        addHierarchyListener(new HierarchyListener() {
            public void hierarchyChanged(HierarchyEvent evt)
            {
                if (evt.getChanged() != View.this || !isVisible()
                        || (evt.getChangeFlags() & HierarchyEvent.SHOWING_CHANGED) == 0)
                    return;

                activate();

                if (task==null)
                    return;

                try
                {
                    task.run();
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
                task = null;
            }
        });
    }

    public void init()
    {
        content = initContentPane();
        this.add(content, BorderLayout.CENTER);
    }

    protected void executeWhenActive(Runnable task)
    {
        if (isVisible())
        {
            task.run();
            return;
        }
        // this is not supposed to happen, but in case we want to be notified that this method
        // must support queuing
        // there should be at most one user notification when view becomes active
        if (this.task!=null) new Exception("Only one task allowed in queue!!").printStackTrace();

        this.task = task;
    }

    protected boolean isTaskWaitingForExecution() {
        return task!=null;
    }

    public final String getDisplayName()
    {
        return header.getTitle();
    }

    public final Icon getDisplayIcon()
    {
        return displayIcon;
    }

    public boolean isReadyForViewChange()
    {
        return true;
    }

    protected abstract JComponent initContentPane();

    protected abstract void activate();
}
